/*
 . . .
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h> /*F_CPU = 4000000; _delay_ms max = 65,535ms; _delay_us max = 192us */

#include "tr71.h"

/*
4000000 meg 
*/


ISR(TIMER2_OVF_vect){
	if ( --time[_led] == 0 )	time_flag|= BIT(_led);
	if ( --time[_01sek] == 0 )	time_flag|= BIT(_01sek);
}



void avr_init(void)
{

	TCCR2=BIT(CS20);
	TIMSK=BIT(TOIE2);//BIT(TOIE0)|

	PORTDRIVE=swt_avto_timer;
	DDRZN|=BITZN;
	DDRLED=0xFF;
	
	DDRSERIAL|=BIT(STORAGE);
	sei();
    return;
}
//-------------------EEPROM-------------------------
static unsigned char EEPROM_read(unsigned int uiAddress)
{
	while(EECR & (1<<EEWE));
	EEAR = uiAddress;
	EECR |= (1<<EERE);
	return EEDR;
}

void EEPROM_write(unsigned int uiAddress, unsigned char ucData)
{
	if ( EEPROM_read(uiAddress)!=ucData){
		while(EECR & (1<<EEWE));
		EEAR = uiAddress;
		EEDR = ucData;
		cli();
		EECR |= (1<<EEMWE);
		EECR |= (1<<EEWE);
		sei();
		}
}

static void save_struct_eep(void)
{
	uint8_t * a= (uint8_t *)&Tim;
	Tim.crc=0;
		for(uint16_t j=0, i=EEP_ADR; j<sizeof(Tim); i++, j++, a++)
			{
			if (j<sizeof(Tim)-2) Tim.crc+=*a;
			EEPROM_write( i, *a);
			}
		eeprom_eer=0;
}

static void read_struct_eep(void)
{
	uint16_t crc=0;
	uint8_t *a= (uint8_t *)&Tim;
	 	for(uint16_t j=0, i=EEP_ADR; j<sizeof(Tim); i++, j++, a++)
			{*a=EEPROM_read(i);
			if (j<sizeof(Tim)-2) crc+=*a;
			}
		if ( crc!= Tim.crc ) eeprom_eer=1;
}
//-------------END------EEPROM----------------------

//**********1wire************

static inline void pullup_on(void){
	PORT_1wire|=pwire0;
	PORT_1wire|=pwire1;
	DDR_1wire|=pwire0;
	DDR_1wire|=pwire1;
}

void pullup_off(void){
	DDR_1wire&=~pwire0;
	DDR_1wire&=~pwire1;
	PORT_1wire&=~pwire0;
	PORT_1wire&=~pwire1;
}

unsigned char readbit(void){
unsigned char i;
	cli();
	DDR_1wire|=out_1wire;//line down
	asm("nop");
	asm("nop");
	asm("nop");
	asm("nop");//1.25us
	DDR_1wire&=~out_1wire;//line up through 1.25us
	_delay_us(11);
	i=PIN_1wire;
	sei();
	_delay_us(47);
	return i;
}

void readbyte(unsigned char *pw0, unsigned char *pw1) {
unsigned char i,r,p;
pullup_off();
*pw0=0;
*pw1=0;
for(i=0, r=1; i<8; i++){
	p=readbit();
	if ( p & pwire0 ) *pw0|=r;
	if ( p & pwire1 ) *pw1|=r;
	r<<=1;	
	}
pullup_on();
}

void writebit0(void){
	cli();
	DDR_1wire|=out_1wire;
	_delay_us(60);
	DDR_1wire&=~out_1wire;
	sei();
}

void writebit1(void){
	cli();
	DDR_1wire|=out_1wire;
	_delay_us(3);
	DDR_1wire&=~out_1wire;
	sei();
	_delay_us(58);
}

void writebyte(unsigned char byte){
unsigned char i;
	pullup_off();
	for(i=0; i<8; i++){
		if (byte&BIT(0))
			writebit1();
			else
			writebit0();
		byte>>=1;
		}
	pullup_on();
}

void detectPresence(void){
unsigned char i;
	pullup_off();
	out_1wire=pwire0|pwire1;
	
	DDR_1wire|=pwire0;
	DDR_1wire|=pwire1;
	_delay_ms(0.48);
	
	cli();
	DDR_1wire&=~pwire0;
	DDR_1wire&=~pwire1;
	_delay_us(60);
	i=PIN_1wire;
	sei();
	
	if ( i & pwire0 ) {out_1wire&=~pwire0; state_led &= ~DRIVE_TEN_1;}
	if ( i & pwire1 ) {out_1wire&=~pwire1; state_led &= ~DRIVE_TEN_2;}
	
	if ( out_1wire ){
		_delay_ms(0.42);
		i=PIN_1wire;
		if (!( i & pwire0 )) {out_1wire&=~pwire0; state_led &= ~DRIVE_TEN_1;}
		if (!( i & pwire1 )) {out_1wire&=~pwire1; state_led &= ~DRIVE_TEN_2;}
		}
}

//***************************
void scratchpad_save(void){
	detectPresence();
	if (out_1wire){
		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0x4E);//WRITE SCRATCHPAD [4Eh]
		writebyte(0);//	TH
		writebyte(0);//	TL
		writebyte(0x7F);//	20 Statys 10bit //writebyte(0x60);//	Statys 12bit byte 4 Configuration Register* 0x7F - 12 bit 750mSek; 1F-9bit 94msek
		detectPresence();
		if (out_1wire){// EEPROM
			writebyte(0xCC);//SKIP ROM [CCh]
			writebyte(0x48);//COPY SCRATCHPAD [48h]
			}
		}
}
//*******end***1wire************


//**************************************************
void format_t(unsigned char p){
int t_point;
	t_point=(unsigned int)(tempir[p*2+1]<<8);//msb
	t_point|=tempir[p*2];
	Temperature[p]=t_point*5/8;
}	

void receive_t(unsigned char *t){

	if ( state_wire==0 )
	{
	detectPresence();
	state_wire=1;
	return;
	}
	
	if ( state_wire==1 )
	{
	if (out_1wire){//
		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0x44);//CONVERT T [44h]
		pullup_on();
		time_wire=100;
		state_wire=2;
		} else state_wire=0;
	return;
	}
	
	if ( state_wire==2 )
	{
	if (!time_wire) state_wire=3;
	return;
	}

	if ( state_wire==3 )
	{
	detectPresence();
	state_wire=4;
	return;
	}
	if ( state_wire==4 )
	{
	if (out_1wire){
		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0xBE);//READ SCRATCHPAD [BEh]
		state_wire=5;
		} else state_wire=0;
	return;
	}	
	if ( state_wire==5 )
	{
	readbyte(t  ,t+2);
	readbyte(t+1,t+3);

	format_t(0);
	format_t(1);
	
	if (++delay_output>=Tim.delay_output+1)// :    *   (10)
	{
	delay_output=0;
	//--------------------------------------------------------	
	if ( pwire0 & out_1wire ){
		if (Temperature[0] <= Tim.Porog[0]) state_led |= DRIVE_TEN_1;
		else
		if (Temperature[0] >= Tim.Porog[1]) state_led &= ~DRIVE_TEN_1;
		}
	else state_led &= ~DRIVE_TEN_1;	
		
	if ( pwire1 & out_1wire ){
		if (Temperature[1] <= Tim.Porog[2]) state_led |= DRIVE_TEN_2;
		else
		if (Temperature[1] >= Tim.Porog[3]) state_led &= ~DRIVE_TEN_2;
		}
	else state_led &= ~DRIVE_TEN_2;
	
	if ( Temperature[0] >= Tim.Porog[1]+2 ) state_led|=ohologden_0; 
	else 
	if ( Temperature[0] <= Tim.Porog[1]-2 ) state_led&=~ohologden_0;
	//--------------------------------------------------------
	}
	
	if ( (Temperature[0] > Tim.Porog[0]) && (Temperature[0] < Tim.Porog[1]) ) state_led|=t_norma_0; else state_led&=~t_norma_0;
	if ( (Temperature[1] > Tim.Porog[2]) && (Temperature[1] < Tim.Porog[3]) ) state_led|=t_norma_1; else state_led&=~t_norma_1;
	
	state_wire=0;
	return;
	}
}

//**************************************
void result_in_bcd(int result)
{//0000 0000 0000 (1)0000  // (2)0000 (3)0000 (4)0000 (5)0000 hi2 hi3 low4 low5
    unsigned char dig_out=0;
	bcd_num1=0;
	bcd_num2=0;
	bcd_num3=0;
	bcd_num4=0;

	while (result >= 1000) {
	 result -= 1000;
 	 dig_out ++;
	}
	bcd_num1=dig_out;
    dig_out = 0;
	while (result >= 100)  {
 	 result -= 100;
	 dig_out ++;
	 }
	bcd_num2=dig_out;
	dig_out = 0;
	while (result >= 10)   {
	 result -= 10;
	 dig_out ++;
	 }
	bcd_num3=dig_out;
	bcd_num4=result;
}

//**************************************
void t_in_bcd(signed int t){
unsigned char znak=0;

	if ( t<0 ){
		znak=1;
		result_in_bcd( t * -1 );
		}
	else 
		result_in_bcd( t );
	
	if ( bcd_num2 ){ 
		lcd_buffer[1]=font[bcd_num2]; 
		if ( znak ) lcd_buffer[0]=font[f_seg_g];
		}
		else
		if ( znak ) lcd_buffer[1]=font[f_seg_g];
	lcd_buffer[2]=font[bcd_num3]|font[tochka]; 
	lcd_buffer[3]=font[bcd_num4]; 
}

//*************************************************************
void data_shift(unsigned char data)
{
//   ,    8 . 
unsigned char i=0; 
	for(i=8; i ; i--){  
		if (data & BIT(7) )  PORTLED|=BIT(DATA);	//   
                    else   PORTLED&=~BIT(DATA);
		data<<=1;
		PORTLED|=BIT(SHIFT); //  
		PORTLED&=~BIT(SHIFT);
		} 
}
//*************************************************************
void state_led_out(void)
{
 PORTLED&=~(BIT(DATA)|BIT(SHIFT));
 data_shift(state_led);//      
 PORTSERIAL|=BIT(STORAGE);//    Z  
 PORTSERIAL&=~BIT(STORAGE);  
 PORTSERIAL|=BIT(STORAGE);
}
//*************************************************************
//-------------------------------------
void skan_key(void)
{
unsigned char pin=0;
	key=0;
	//PORTZN &= ~BITZN;
#ifdef katod
	PORTZN |= BITZN;
#else
	PORTZN &= ~BITZN;	
#endif
	DDRLED=BIT(7);
	PORTLED=(unsigned char)~BIT(7);
	_delay_us(30);
	pin=PINLED|BIT(7);
	if ( pin!= 0xFF ){
		time_view_zadanie=700;//0.05
		if (++anti_dr==10)
			{
			key= ~pin;
			}
		if (anti_dr==btn_speed){
			anti_dr=0;
			if (n_btn_sp<10){
				if (++n_btn_sp==2) btn_speed=75;
				if (n_btn_sp==4) btn_speed=25;
				if (n_btn_sp==7) btn_speed=11;
				}
			}
		}
		else{
		btn_speed=200;
		n_btn_sp=0;
		anti_dr=0;
		}
	DDRLED=0xFF;
}

//-------------------------------------
void display(void)
{

	state_led_out();
#ifdef katod
	PORTZN |= BITZN;
	
	if ( znmesto==0 )	PORTZN &= ~ZN0;//PORTZN &= ~ZN0;//
	else
	if ( znmesto==1 )	PORTZN &= ~ZN1;//PORTZN &= ~ZN1;//
	else
	if ( znmesto==2 )	PORTZN &= ~ZN2;
	else
	if ( znmesto==3 )	PORTZN &= ~ZN3;
	PORTLED=lcd_buffer[znmesto];
#else
	PORTZN &= ~BITZN;

	if ( znmesto==0 )	PORTZN |= ZN0;//PORTZN &= ~ZN0;//
	else
	if ( znmesto==1 )	PORTZN |= ZN1;//PORTZN &= ~ZN1;//
	else
	if ( znmesto==2 )	PORTZN |= ZN2;
	else
	if ( znmesto==3 )	PORTZN |= ZN3;
	PORTLED=~lcd_buffer[znmesto];
#endif
}
//-------------------------------------
int16_t incdec(int16_t p,int16_t Bottom,int16_t Top){
	if ( key & btn_plus ){
		if (p<Top) p++; else p=Bottom;
		time[_viewp]=5;
		}
	if ( key & btn_minus ){
		if (p>Bottom) p--; else p=Top;
		time[_viewp]=5;
		}
	return p;
}

void key_action(void)
{

if (key)
	{
	if (view_zadanie)
	{
	if (view_tim_drive==0){
		Tim.Porog[view_zadanie-1]=incdec(Tim.Porog[view_zadanie-1], -537, 998);
		}
		else{
			if (view_tim_drive<3){
				Tim.Timehhmm[view_tim_drive-1]=incdec(Tim.Timehhmm[view_tim_drive-1], 0, 59);
			}
			else
			if (view_tim_drive==3)
				Tim.TimeWork=incdec(Tim.TimeWork, 0, 999);
			else
			if (view_tim_drive<6)
				Tim.TimeDsView[view_tim_drive-4]=incdec(Tim.TimeDsView[view_tim_drive-4], 0, 90);
			else
			if (view_tim_drive==6)
				Tim.delay_output=incdec(Tim.delay_output, 0, 90);
		}
	}
	
	if ( key & btn_set )
		{
		if (view_tim_drive){
			view_tim_drive=0;
			view_zadanie=0;
			}
		if (++view_zadanie==5) {view_zadanie=0; save_struct_eep();}
		time_flag |= BIT(_viewp);
		time[_viewp]=100;
		}
	
	if ( key & btn_set_timer )
		{
		if (++view_tim_drive==7) {view_tim_drive=0; save_struct_eep();}
		view_zadanie=view_tim_drive;
		time_flag |= BIT(_viewp);
		time[_viewp]=100;
		}
	if ( key & btn_start_timer ){
		taimer[0]=(unsigned int)Tim.Timehhmm[0]*60+Tim.Timehhmm[1];
		if (time_flag & BIT(_timer_start)) 
			time_flag &=~ BIT(_timer_start);
			else {
			time_flag |= BIT(_timer_start); 
			tim_minuta=0;
			}
		state_led &= ~DRIVE_TIMER;
		time_flag &=~ BIT(_timer_on);
		}
	}
}

uint8_t presence_ds(uint8_t pin_wire){
uint8_t p=1;
	if ( !(pin_wire & out_1wire) ){
	 	lcd_buffer[0]	=font[f_seg_g];
		lcd_buffer[1]	=font[f_seg_g];
		lcd_buffer[2]	=font[f_seg_g];
		lcd_buffer[3]	=font[f_seg_g];
		p=0;
		}
	return p;
}
//-------------------------------------
void data_led(void)
{
unsigned char i;

for(i=0;i<4;i++)
	lcd_buffer[i]=0x0;
	
	if (view_zadanie)
	{//   
	if ( time_flag & BIT(_viewp) )
		{
		if(!view_tim_drive){
			lcd_buffer[1]=font[f_P];
			lcd_buffer[2]=font[view_zadanie];
			}
			else{
			if ( view_tim_drive<3 )
				{
				if ( view_tim_drive & BIT(0) ){
					lcd_buffer[0]=font[4];
					lcd_buffer[1]=font[f_a];
					lcd_buffer[2]=font[f_C];
					}else{
					lcd_buffer[0]=font[f_n];
					lcd_buffer[1]=font[f_n];
					lcd_buffer[2]=font[f_t]|font[tochka];
					}
				}
				else
				if (view_tim_drive==3)
				{
				lcd_buffer[0]=font[5];
				lcd_buffer[1]=font[f_e];
				lcd_buffer[2]=font[f_C]|font[tochka];
				}
				else
				if (view_tim_drive<6)
				{
				lcd_buffer[0]=font[f_t];
				lcd_buffer[2]=font[f_d];
				lcd_buffer[3]=font[view_tim_drive-3];
				}
				else
				{
				lcd_buffer[1]=font[f_d];
				lcd_buffer[2]=font[f_P];
				lcd_buffer[3]=font[f_e];
				}
			}
		}
		else 
		{
		if ( !view_tim_drive ) 
		t_in_bcd(Tim.Porog[view_zadanie-1]);
		else
			{
			if ( view_tim_drive<3 ){
				result_in_bcd(Tim.Timehhmm[view_tim_drive-1]);
				lcd_buffer[0]=font[view_tim_drive]; 
				lcd_buffer[2]=font[bcd_num3]; 
				lcd_buffer[3]=font[bcd_num4];
				} 
				else
			if ( view_tim_drive==3 ){
				result_in_bcd(Tim.TimeWork);
				lcd_buffer[1]=font[bcd_num2]; 
				lcd_buffer[2]=font[bcd_num3]; 
				lcd_buffer[3]=font[bcd_num4];
				}
				else
			if ( view_tim_drive<6 ){
				result_in_bcd(Tim.TimeDsView[view_tim_drive-4]);
				lcd_buffer[0]=font[view_tim_drive-3]; 
				lcd_buffer[2]=font[bcd_num3]; 
				lcd_buffer[3]=font[bcd_num4];
				}
				else{
				result_in_bcd(Tim.delay_output); 
				lcd_buffer[2]=font[bcd_num3]; 
				lcd_buffer[3]=font[bcd_num4];
				}
			}
		}
	}
	else
	{//  
	if ( eeprom_eer==0 )
		{
		if ( time_flag & BIT(_view_ds_0) ){ 
			if (presence_ds(pwire0)) t_in_bcd(Temperature[0]);
			state_led|=view_ds_0;
			}
			else{
			if (presence_ds(pwire1)) t_in_bcd(Temperature[1]);
			state_led&=~view_ds_0;
			}
		}
		else
		{
		lcd_buffer[1]=font[f_e]; 
		lcd_buffer[2]=font[f_e];
		lcd_buffer[3]=font[f_P]; 
		}
	
	//  (74595)
	state_led&=~dr_timer_on;
	if ( tim_1sek<51 )
		if ( time_flag & BIT(_timer_start) ) state_led|=dr_timer_on; 
	//-------------	
	}
/*
state_led     ,        , 
       . display
*/
}
//-------------------------------------



int main(void)
{
   
	avr_init();	
	read_struct_eep();
	
	taimer[0]=(unsigned int)Tim.Timehhmm[0]*60+Tim.Timehhmm[1];
	taimer[1]=Tim.TimeWork;
	time_view_ds=Tim.TimeDsView[0];
	time_view_ds=Tim.TimeDsView[1];
	
		
	detectPresence();
	if (out_1wire){
		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0xBE);//READ SCRATCHPAD [BEh]
		readbyte(&tempir[0],&tempir[0]);//LSB
		readbyte(&tempir[0],&tempir[0]);//MSB
		readbyte(&tempir[0],&tempir[0]);
		readbyte(&tempir[0],&tempir[0]);
		readbyte(&tempir[0],&tempir[1]);
		//byte 4 Configuration Register* 0x1F-9bit 94msek; 3F-10bit 188msek; 0x5F-11bit 375msek; 0x7F - 12 bit 750mSek;   		
		if ( 	((tempir[0]!=0x7F)&&(pwire0 & out_1wire)) 
			||	((tempir[1]!=0x7F)&&(pwire1 & out_1wire)) ){
			scratchpad_save();//10ms
			_delay_ms(10);
			}
		tempir[0]=0;
		}
	
while(1)
	{
	
	if ( time_flag & BIT(_led) )// 
		{
		time_flag &= ~BIT(_led);
		time[_led]=70;//~0.00
		data_led();
		skan_key();
		key_action();
		display();
		if ( ++znmesto==4 )
			{
			znmesto=0;
			}
		receive_t(&tempir[0]);
		}
	if ( time_flag & BIT(_01sek) )
		{
		time_flag &= ~BIT(_01sek);
		time[_01sek]=156;//~0,01
		time_wire--;
		if (--time_view_zadanie==0) view_zadanie=0;
		if (--time[_viewp]==0) time_flag &= ~BIT(_viewp);
		//*
		if (--tim_1sek==0)
			{
			tim_1sek=100;
			
			if ( --time_view_ds == 0 ) 
				{
				time_flag^=BIT(_view_ds_0);
				
				if ( time_flag & BIT(_view_ds_0) )
					time_view_ds=Tim.TimeDsView[0];
					else
					time_view_ds=Tim.TimeDsView[1];
				
				if (!time_view_ds) 
					{
					time_flag^=BIT(_view_ds_0);
					time_view_ds=1;
					}
				}
			
			if ( !(PINDRIVE & swt_avto_timer) ) time_flag |= BIT(_timer_start);
			
			if ( time_flag & BIT(_timer_on) ){
				if ( taimer[1] )
					if (--taimer[1]==0){
						time_flag &=~ ( BIT(_timer_on)|BIT(_timer_start) );
						taimer[0]=(unsigned int)Tim.Timehhmm[0]*60+Tim.Timehhmm[1];
						tim_minuta=0;
						state_led &= ~DRIVE_TIMER;
						}
				}
				else
				if (++tim_minuta==60)//60
				{
				tim_minuta=0;
				if ( (time_flag & BIT(_timer_on))==0 )
					{
					if ( time_flag & BIT(_timer_start) )
						if (--taimer[0]==0){
							time_flag|=BIT(_timer_on);
							taimer[1]=Tim.TimeWork;
							state_led |= DRIVE_TIMER;
							}	
					}
				}
			}
			//*/
		}
	}

}

